@@ -1,14 +1,20 @@
-#ifdef _CH_
-#pragma package <opencv>
-#endif
+#include "opencv2/objdetect/objdetect.hpp"
+#include "opencv2/highgui/highgui.hpp"
 
-#define CV_NO_BACKWARD_COMPATIBILITY
-
-#ifndef _EiC
-#include "cv.h"
-#include "highgui.h"
+#include <ctype.h>
 #include <stdio.h>
-#endif
+
+static void help(void)
+{
+    printf("\n This sample demonstrates cascade's convertation \n"
+    "Usage:\n"
+    "./convert_cascade --size=\"<width>x<height>\"<convertation size> \n"
+    "                   input_cascade_path \n"
+    "                   output_cascade_filename\n"
+    "Example: \n"
+    "./convert_cascade --size=640x480 ../../opencv/data/haarcascades/haarcascade_eye.xml ../../opencv/data/haarcascades/test_cascade.xml \n"
+    );
+}
 
 int main( int argc, char** argv )
 {
@@ -17,9 +23,11 @@ int main( int argc, char** argv )
     CvHaarClassifierCascade* cascade = 0;
     CvSize size;
 
+    help();
+
     if( argc != 4 || strncmp( argv[1], size_opt, strlen(size_opt) ) != 0 )
     {
-        printf( "Usage: convert_cascade --size=\"<width>x<height>\" input_cascade_path output_cascade_filename\n" );
+    	help();
         return -1;
     }
 
